import java.io.*;
import java.net.*;
import java.util.*;

public class HiPixProxyServer
{
	public static final int STANDARD_TIMEOUT = 120000; // 2 minutes
	public static final String CHARSET = "ISO8859_1";
	private static final int DOS_BYTE_LIMIT = 10000;

	public static void main(String[] args)
	{
		java.net.ServerSocket ss = null;
		try
		{
			ss = new java.net.ServerSocket(6969);
		}
		catch (Exception e)
		{
			System.out.println("Error creating HiPixProxyServer socket:" + e);
			return;
		}
		System.out.println("HiPixProxyServer launched on port 6969");
		java.io.File currFile = null;
		while (true)
		{
			java.net.Socket mySock = null;
			java.net.Socket outSock = null;
			java.io.DataOutputStream outStream = null;
			java.io.DataInputStream inStream = null;
			java.io.DataOutputStream outStream2 = null;
			java.io.DataInputStream inStream2 = null;
			try
			{
				mySock = ss.accept();
				System.out.println("HiPixProxyServer received connection:" + mySock);
				mySock.setSoTimeout(STANDARD_TIMEOUT);
				outStream = new java.io.DataOutputStream(new java.io.BufferedOutputStream(mySock.getOutputStream()));
				inStream = new java.io.DataInputStream(new java.io.BufferedInputStream(mySock.getInputStream()));
				String pass = readLineBytes(inStream);
				// valid password, accept the encoding command
				outStream.write("OK\r\n".getBytes());
				outStream.flush();
				String str = readLineBytes(inStream);
				System.out.println("HiPixProxyServer Recvd:" + str);
				
				// Create the connection to the HiPix application
				outSock = new java.net.Socket("localhost", 9000);
				outSock.setSoTimeout(STANDARD_TIMEOUT);
				outStream2 = new java.io.DataOutputStream(new java.io.BufferedOutputStream(outSock.getOutputStream()));
				inStream2 = new java.io.DataInputStream(new java.io.BufferedInputStream(outSock.getInputStream()));
				// First it should say connected
				String str2 = quickRead(outSock, inStream2);
				System.out.println("Received from HiPix app:" + str2);
				if (str.equals("STOP"))
				{
					outStream2.write("$STOP".getBytes());
					outStream2.flush();
					str2 = quickRead(outSock, inStream2);
					System.out.println("Received response from HiPix app:" + str2);
					outStream.write("OK\r\n".getBytes());
				}
				else if (str.startsWith("START "))
				{
					java.util.StringTokenizer toker = new java.util.StringTokenizer(str.substring(6), "|");
					final String source = toker.nextToken();
					final String channel = toker.nextToken();
					final long endTime = System.currentTimeMillis() + Long.parseLong(toker.nextToken());
					String filename = toker.nextToken()/* + ".ts"*/;
					final String encoding = toker.nextToken();
					currFile = new java.io.File(filename);
					
					// Stop anything currently running
					System.out.println("Stopping current recording on hipix for safety...");
					outStream2.write("$STOP".getBytes());
					outStream2.flush();
					str2 = quickRead(outSock, inStream2);
					System.out.println("Received response from HiPix app:" + str2);
					
					boolean isDTV = channel.indexOf('-') != -1;
					isDTV = true;
					System.out.println("Attempting to put HiPix into " + (isDTV ? "ATSC" : "NTSC") + " mode");
					outStream2.write((isDTV ? "$ATSC" : "$NTSC").getBytes());
					outStream2.flush();
					str2 = quickRead(outSock, inStream2);
					System.out.println("Received response from HiPix app:" + str2);
					
					System.out.println("Setting channel number on HiPix");
					outStream2.write(("$CHANNEL " + channel).getBytes());
					outStream2.flush();
					str2 = quickRead(outSock, inStream2);
					System.out.println("Received response from HiPix app:" + str2);
					
					System.out.println("Starting record to file " + filename + " with HiPix");
					outStream2.write(("$RECORD " + filename).getBytes());
					outStream2.flush();
					str2 = quickRead(outSock, inStream2);
					System.out.println("Received response from HiPix app:" + str2);
					
					outStream.write("OK\r\n".getBytes());
				}
				else if (str.equals("GET_SIZE"))
				{
					outStream.write(((currFile == null ? "0" : Long.toString(currFile.length())) + "\r\n").getBytes());
				}
				else if (str.equals("NOOP"))
				{
					outStream.write("OK\r\n".getBytes());
				}
				else
				{
					outStream.write(("ERROR Unrecognized command of:" + str + "\r\n").getBytes());
				}
				outStream.flush();
			}
			catch (Exception e)
			{
				System.out.println("Error with EncodingServer socket:" + e);
				if (outStream != null)
				{
					try
					{
						outStream.write(("ERROR " + e + "\r\n").getBytes());
						outStream.flush();
					}
					catch (Exception e1)
					{}
				}
			}
			finally
			{
				try
				{
					if (inStream != null)
						inStream.close();
					if (outStream != null)
						outStream.close();
					if (mySock != null)
						mySock.close();
					if (inStream2 != null)
						inStream2.close();
					if (outStream2 != null)
						outStream2.close();
					if (outSock != null)
						outSock.close();
				}catch (Exception e){}
			}
		}
	}
	

	// Reads a line from a binary stream, terminates on '\r\n' or '\n'.
	// Yeah, it should also terminate on \r itself, but the transfer
	// protocol this is used for requires \r\n termination anyways.
	public static String readLineBytes(java.io.DataInput inStream)
		throws java.io.InterruptedIOException, java.io.IOException
	{
		synchronized (inStream)
		{
			StringBuffer result = new StringBuffer();
			byte currByte = inStream.readByte();
			int readTillPanic = DOS_BYTE_LIMIT;
			while (true)
			{
				if (currByte == '\r')
				{
					currByte = inStream.readByte();
					if (currByte == '\n')
					{
						return result.toString();
					}
					result.append('\r');
				}
				else if (currByte == '\n')
				{
					return result.toString();
				}
				result.append((char)currByte);
				currByte = inStream.readByte();
				if (readTillPanic-- < 0)
				{
					throw new java.io.IOException("TOO MANY BYTES RECIEVED FOR A LINE, BAILING TO PREVENT DOS ATTACK");
				}
			}
		}
	}
	
	public static String quickRead(java.net.Socket theSock, java.io.DataInput inStream) 
		throws java.io.InterruptedIOException, java.io.IOException
	{
		synchronized (inStream)
		{
			StringBuffer result = new StringBuffer();
			byte currByte = inStream.readByte();
			int oldTimeout = theSock.getSoTimeout();
			theSock.setSoTimeout(50);
			while (true)
			{
				result.append((char)currByte);
				try
				{
					currByte = inStream.readByte();
				}
				catch (java.io.InterruptedIOException e)
				{
					theSock.setSoTimeout(oldTimeout);
					return result.toString();
				}
			}
		}
	}
	
}